@@ -553,11 +553,21 @@ static int HX_getopt_long(const char *cur, struct HX_getopt_vars *par)
 		return -errno;
 	value = strchr(key, '=');
 	if (value == nullptr) {
-		/* Cannot happen because state is always !S_TWOLONG */
+		/*
+		 * Cannot happen because state is always !S_TWOLONG, but make
+		 * static analyzers happy.
+		 */
 		free(key);
 		return -EINVAL;
 	}
-	*value++ = '\0';
+	*value = '\0';
+	value = strchr(cur, '=');
+	if (value == nullptr) {
+		/* Cannot happen either */
+		free(key);
+		return -EINVAL;
+	}
+	++value;
 	par->cbi.current = lookup_long_pfx(par->cbi.table, key + 2);
 	if (par->cbi.current == &HXopt_ambig_prefix) {
 		ret = HX_getopt_error(HXOPT_E_AMBIG_PREFIX, key, par->flags);
@@ -579,6 +589,7 @@ static int HX_getopt_long(const char *cur, struct HX_getopt_vars *par)
 		return ret;
 	}
 
+	free(key);
 	par->cbi.flags    = HXOPTCB_BY_LONG;
 	par->cbi.data     = value;
 	/*
@@ -587,7 +598,6 @@ static int HX_getopt_long(const char *cur, struct HX_getopt_vars *par)
 	 * function.
 	 */
 	ret = do_assign_front(par);
-	free(key);
 	if (ret < 0)
 		return ret;
 	return HXOPT_S_NORMAL | HXOPT_I_ADVARG;
@@ -716,7 +726,8 @@ EXPORT_SYMBOL int HX_getopt6(const struct HXoption *table, int argc,
 
 	if (!(ps.flags & HXOPT_ANY_ORDER) && posix_me_harder())
 		ps.flags |= HXOPT_RQ_ORDER;
-	for (const char *op0 = opt[0]; argc > 0; ) {
+	const char *op0 = argc > 0 ? opt[0] : nullptr;
+	while (argc > 0) {
 		if (state == HXOPT_S_TWOLONG)
 			state = HX_getopt_twolong(op0, argc > 1 ? opt[1] : nullptr, &ps);
 		else if (state == HXOPT_S_LONG)
@@ -742,11 +753,14 @@ EXPORT_SYMBOL int HX_getopt6(const struct HXoption *table, int argc,
 				goto out;
 		}
 		if (state & HXOPT_I_ADVARG) {
-			op0 = *++opt;
 			--argc;
+			if (argc > 0)
+				op0 = *++opt;
 		} else if (state & HXOPT_I_ADVARG2) {
-			op0 = *(opt += 2);
+			opt += 2;
 			argc -= 2;
+			if (argc > 0)
+				op0 = *opt;
 		} else if (state & HXOPT_I_ADVCHAR) {
 			++op0;
 		}
